@@ -1,8 +1,7 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*- */
 /*
  * This file is part of GtkSourceView
  *
- * Copyright (C) 2014, 2015 - Sébastien Wilmet <swilmet@gnome.org>
+ * Copyright 2014, 2015 - Sébastien Wilmet <swilmet@gnome.org>
  *
  * GtkSourceView is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,11 +17,9 @@
  * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
-#include "gtksourcefile.h"
+#include "gtksourcefile-private.h"
 #include "gtksourceencoding.h"
 #include "gtksource-enumtypes.h"
 
@@ -39,6 +36,12 @@
  * properties). On a successful load or save operation, the #GtkSourceFile
  * properties are updated. If an operation fails, the #GtkSourceFile properties
  * have still the previous valid values.
+ *
+ * <warning>
+ * This class is no longer maintained, patches are not accepted. There is a
+ * better implementation in the
+ * <ulink url="https://wiki.gnome.org/Projects/Tepl">Tepl</ulink> library.
+ * </warning>
  */
 
 enum
@@ -48,10 +51,11 @@ enum
 	PROP_ENCODING,
 	PROP_NEWLINE_TYPE,
 	PROP_COMPRESSION_TYPE,
-	PROP_READ_ONLY
+	PROP_READ_ONLY,
+	N_PROPS
 };
 
-struct _GtkSourceFilePrivate
+typedef struct
 {
 	GFile *location;
 	const GtkSourceEncoding *encoding;
@@ -65,78 +69,73 @@ struct _GtkSourceFilePrivate
 	/* Last known modification time of 'location'. The value is updated on a
 	 * file loading or file saving.
 	 */
-	GTimeVal modification_time;
+	gint64 modification_time;
 
 	guint modification_time_set : 1;
 
 	guint externally_modified : 1;
 	guint deleted : 1;
 	guint readonly : 1;
-};
+} GtkSourceFilePrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceFile, gtk_source_file, G_TYPE_OBJECT)
 
+static GParamSpec *properties[N_PROPS];
+
 static void
 gtk_source_file_get_property (GObject    *object,
-			      guint       prop_id,
-			      GValue     *value,
-			      GParamSpec *pspec)
+                              guint       prop_id,
+                              GValue     *value,
+                              GParamSpec *pspec)
 {
-	GtkSourceFile *file;
-
-	g_return_if_fail (GTK_SOURCE_IS_FILE (object));
-
-	file = GTK_SOURCE_FILE (object);
+	GtkSourceFile *file = GTK_SOURCE_FILE (object);
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
 
 	switch (prop_id)
 	{
-		case PROP_LOCATION:
-			g_value_set_object (value, file->priv->location);
-			break;
+	case PROP_LOCATION:
+		g_value_set_object (value, priv->location);
+		break;
 
-		case PROP_ENCODING:
-			g_value_set_boxed (value, file->priv->encoding);
-			break;
+	case PROP_ENCODING:
+		g_value_set_boxed (value, priv->encoding);
+		break;
 
-		case PROP_NEWLINE_TYPE:
-			g_value_set_enum (value, file->priv->newline_type);
-			break;
+	case PROP_NEWLINE_TYPE:
+		g_value_set_enum (value, priv->newline_type);
+		break;
 
-		case PROP_COMPRESSION_TYPE:
-			g_value_set_enum (value, file->priv->compression_type);
-			break;
+	case PROP_COMPRESSION_TYPE:
+		g_value_set_enum (value, priv->compression_type);
+		break;
 
-		case PROP_READ_ONLY:
-			g_value_set_boolean (value, file->priv->readonly);
-			break;
+	case PROP_READ_ONLY:
+		g_value_set_boolean (value, priv->readonly);
+		break;
 
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-			break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
 	}
 }
 
 static void
 gtk_source_file_set_property (GObject      *object,
-			      guint         prop_id,
-			      const GValue *value,
-			      GParamSpec   *pspec)
+                              guint         prop_id,
+                              const GValue *value,
+                              GParamSpec   *pspec)
 {
-	GtkSourceFile *file;
-
-	g_return_if_fail (GTK_SOURCE_IS_FILE (object));
-
-	file = GTK_SOURCE_FILE (object);
+	GtkSourceFile *file = GTK_SOURCE_FILE (object);
 
 	switch (prop_id)
 	{
-		case PROP_LOCATION:
-			gtk_source_file_set_location (file, g_value_get_object (value));
-			break;
+	case PROP_LOCATION:
+		gtk_source_file_set_location (file, g_value_get_object (value));
+		break;
 
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-			break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
 	}
 }
 
@@ -144,13 +143,14 @@ static void
 gtk_source_file_dispose (GObject *object)
 {
 	GtkSourceFile *file = GTK_SOURCE_FILE (object);
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
 
-	g_clear_object (&file->priv->location);
+	g_clear_object (&priv->location);
 
-	if (file->priv->mount_operation_notify != NULL)
+	if (priv->mount_operation_notify != NULL)
 	{
-		file->priv->mount_operation_notify (file->priv->mount_operation_userdata);
-		file->priv->mount_operation_notify = NULL;
+		priv->mount_operation_notify (priv->mount_operation_userdata);
+		priv->mount_operation_notify = NULL;
 	}
 
 	G_OBJECT_CLASS (gtk_source_file_parent_class)->dispose (object);
@@ -172,15 +172,14 @@ gtk_source_file_class_init (GtkSourceFileClass *klass)
 	 *
 	 * Since: 3.14
 	 */
-	g_object_class_install_property (object_class,
-					 PROP_LOCATION,
-					 g_param_spec_object ("location",
-							      "Location",
-							      "",
-							      G_TYPE_FILE,
-							      G_PARAM_READWRITE |
-							      G_PARAM_CONSTRUCT |
-							      G_PARAM_STATIC_STRINGS));
+	properties [PROP_LOCATION] =
+		g_param_spec_object ("location",
+		                     "Location",
+		                     "",
+		                     G_TYPE_FILE,
+		                     (G_PARAM_READWRITE |
+		                      G_PARAM_CONSTRUCT |
+		                      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GtkSourceFile:encoding:
@@ -190,14 +189,13 @@ gtk_source_file_class_init (GtkSourceFileClass *klass)
 	 *
 	 * Since: 3.14
 	 */
-	g_object_class_install_property (object_class,
-					 PROP_ENCODING,
-					 g_param_spec_boxed ("encoding",
-							     "Encoding",
-							     "",
-							     GTK_SOURCE_TYPE_ENCODING,
-							     G_PARAM_READABLE |
-							     G_PARAM_STATIC_STRINGS));
+	properties[PROP_ENCODING] =
+		g_param_spec_boxed ("encoding",
+		                    "Encoding",
+		                    "",
+		                    GTK_SOURCE_TYPE_ENCODING,
+		                    (G_PARAM_READABLE |
+		                     G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GtkSourceFile:newline-type:
@@ -206,15 +204,14 @@ gtk_source_file_class_init (GtkSourceFileClass *klass)
 	 *
 	 * Since: 3.14
 	 */
-	g_object_class_install_property (object_class,
-					 PROP_NEWLINE_TYPE,
-					 g_param_spec_enum ("newline-type",
-							    "Newline type",
-							    "",
-							    GTK_SOURCE_TYPE_NEWLINE_TYPE,
-							    GTK_SOURCE_NEWLINE_TYPE_LF,
-							    G_PARAM_READABLE |
-							    G_PARAM_STATIC_STRINGS));
+	properties[PROP_NEWLINE_TYPE] =
+		g_param_spec_enum ("newline-type",
+		                   "Newline type",
+		                   "",
+		                   GTK_SOURCE_TYPE_NEWLINE_TYPE,
+		                   GTK_SOURCE_NEWLINE_TYPE_LF,
+		                   (G_PARAM_READABLE |
+		                    G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GtkSourceFile:compression-type:
@@ -223,15 +220,14 @@ gtk_source_file_class_init (GtkSourceFileClass *klass)
 	 *
 	 * Since: 3.14
 	 */
-	g_object_class_install_property (object_class,
-					 PROP_COMPRESSION_TYPE,
-					 g_param_spec_enum ("compression-type",
-							    "Compression type",
-							    "",
-							    GTK_SOURCE_TYPE_COMPRESSION_TYPE,
-							    GTK_SOURCE_COMPRESSION_TYPE_NONE,
-							    G_PARAM_READABLE |
-							    G_PARAM_STATIC_STRINGS));
+	properties [PROP_COMPRESSION_TYPE] =
+		g_param_spec_enum ("compression-type",
+		                   "Compression type",
+		                   "",
+		                   GTK_SOURCE_TYPE_COMPRESSION_TYPE,
+		                   GTK_SOURCE_COMPRESSION_TYPE_NONE,
+		                   (G_PARAM_READABLE |
+		                    G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GtkSourceFile:read-only:
@@ -241,24 +237,25 @@ gtk_source_file_class_init (GtkSourceFileClass *klass)
 	 *
 	 * Since: 3.18
 	 */
-	g_object_class_install_property (object_class,
-					 PROP_READ_ONLY,
-					 g_param_spec_boolean ("read-only",
-							       "Read Only",
-							       "",
-							       FALSE,
-							       G_PARAM_READABLE |
-							       G_PARAM_STATIC_STRINGS));
+	properties [PROP_READ_ONLY] =
+		g_param_spec_boolean ("read-only",
+		                      "Read Only",
+		                      "",
+		                      FALSE,
+		                      (G_PARAM_READABLE |
+		                       G_PARAM_STATIC_STRINGS));
+
+	g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
 static void
 gtk_source_file_init (GtkSourceFile *file)
 {
-	file->priv = gtk_source_file_get_instance_private (file);
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
 
-	file->priv->encoding = NULL;
-	file->priv->newline_type = GTK_SOURCE_NEWLINE_TYPE_LF;
-	file->priv->compression_type = GTK_SOURCE_COMPRESSION_TYPE_NONE;
+	priv->encoding = NULL;
+	priv->newline_type = GTK_SOURCE_NEWLINE_TYPE_LF;
+	priv->compression_type = GTK_SOURCE_COMPRESSION_TYPE_NONE;
 }
 
 /**
@@ -284,20 +281,22 @@ gtk_source_file_new (void)
  */
 void
 gtk_source_file_set_location (GtkSourceFile *file,
-			      GFile         *location)
+                              GFile         *location)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_if_fail (GTK_SOURCE_IS_FILE (file));
 	g_return_if_fail (location == NULL || G_IS_FILE (location));
 
-	if (g_set_object (&file->priv->location, location))
+	if (g_set_object (&priv->location, location))
 	{
-		g_object_notify (G_OBJECT (file), "location");
+		g_object_notify_by_pspec (G_OBJECT (file), properties[PROP_LOCATION]);
 
 		/* The modification_time is for the old location. */
-		file->priv->modification_time_set = FALSE;
+		priv->modification_time_set = FALSE;
 
-		file->priv->externally_modified = FALSE;
-		file->priv->deleted = FALSE;
+		priv->externally_modified = FALSE;
+		priv->deleted = FALSE;
 	}
 }
 
@@ -311,21 +310,25 @@ gtk_source_file_set_location (GtkSourceFile *file,
 GFile *
 gtk_source_file_get_location (GtkSourceFile *file)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE (file), NULL);
 
-	return file->priv->location;
+	return priv->location;
 }
 
 void
 _gtk_source_file_set_encoding (GtkSourceFile           *file,
-			       const GtkSourceEncoding *encoding)
+                               const GtkSourceEncoding *encoding)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_if_fail (GTK_SOURCE_IS_FILE (file));
 
-	if (file->priv->encoding != encoding)
+	if (priv->encoding != encoding)
 	{
-		file->priv->encoding = encoding;
-		g_object_notify (G_OBJECT (file), "encoding");
+		priv->encoding = encoding;
+		g_object_notify_by_pspec (G_OBJECT (file), properties[PROP_ENCODING]);
 	}
 }
 
@@ -342,21 +345,25 @@ _gtk_source_file_set_encoding (GtkSourceFile           *file,
 const GtkSourceEncoding *
 gtk_source_file_get_encoding (GtkSourceFile *file)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE (file), NULL);
 
-	return file->priv->encoding;
+	return priv->encoding;
 }
 
 void
 _gtk_source_file_set_newline_type (GtkSourceFile        *file,
-				   GtkSourceNewlineType  newline_type)
+                                   GtkSourceNewlineType  newline_type)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_if_fail (GTK_SOURCE_IS_FILE (file));
 
-	if (file->priv->newline_type != newline_type)
+	if (priv->newline_type != newline_type)
 	{
-		file->priv->newline_type = newline_type;
-		g_object_notify (G_OBJECT (file), "newline-type");
+		priv->newline_type = newline_type;
+		g_object_notify_by_pspec (G_OBJECT (file), properties[PROP_NEWLINE_TYPE]);
 	}
 }
 
@@ -370,21 +377,25 @@ _gtk_source_file_set_newline_type (GtkSourceFile        *file,
 GtkSourceNewlineType
 gtk_source_file_get_newline_type (GtkSourceFile *file)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE (file), GTK_SOURCE_NEWLINE_TYPE_DEFAULT);
 
-	return file->priv->newline_type;
+	return priv->newline_type;
 }
 
 void
 _gtk_source_file_set_compression_type (GtkSourceFile            *file,
-				       GtkSourceCompressionType  compression_type)
+                                       GtkSourceCompressionType  compression_type)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_if_fail (GTK_SOURCE_IS_FILE (file));
 
-	if (file->priv->compression_type != compression_type)
+	if (priv->compression_type != compression_type)
 	{
-		file->priv->compression_type = compression_type;
-		g_object_notify (G_OBJECT (file), "compression-type");
+		priv->compression_type = compression_type;
+		g_object_notify_by_pspec (G_OBJECT (file), properties[PROP_COMPRESSION_TYPE]);
 	}
 }
 
@@ -398,9 +409,11 @@ _gtk_source_file_set_compression_type (GtkSourceFile            *file,
 GtkSourceCompressionType
 gtk_source_file_get_compression_type (GtkSourceFile *file)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE (file), GTK_SOURCE_COMPRESSION_TYPE_NONE);
 
-	return file->priv->compression_type;
+	return priv->compression_type;
 }
 
 /**
@@ -423,34 +436,40 @@ gtk_source_file_get_compression_type (GtkSourceFile *file)
  */
 void
 gtk_source_file_set_mount_operation_factory (GtkSourceFile                  *file,
-					     GtkSourceMountOperationFactory  callback,
-					     gpointer                        user_data,
-					     GDestroyNotify                  notify)
+                                             GtkSourceMountOperationFactory  callback,
+                                             gpointer                        user_data,
+                                             GDestroyNotify                  notify)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_if_fail (GTK_SOURCE_IS_FILE (file));
 
-	if (file->priv->mount_operation_notify != NULL)
+	if (priv->mount_operation_notify != NULL)
 	{
-		file->priv->mount_operation_notify (file->priv->mount_operation_userdata);
+		priv->mount_operation_notify (priv->mount_operation_userdata);
 	}
 
-	file->priv->mount_operation_factory = callback;
-	file->priv->mount_operation_userdata = user_data;
-	file->priv->mount_operation_notify = notify;
+	priv->mount_operation_factory = callback;
+	priv->mount_operation_userdata = user_data;
+	priv->mount_operation_notify = notify;
 }
 
 GMountOperation *
 _gtk_source_file_create_mount_operation (GtkSourceFile *file)
 {
-	return (file != NULL && file->priv->mount_operation_factory != NULL) ?
-		file->priv->mount_operation_factory (file, file->priv->mount_operation_userdata) :
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
+	return (file != NULL && priv->mount_operation_factory != NULL) ?
+		priv->mount_operation_factory (file, priv->mount_operation_userdata) :
 		g_mount_operation_new ();
 }
 
 gboolean
 _gtk_source_file_get_modification_time (GtkSourceFile *file,
-					GTimeVal      *modification_time)
+					gint64        *modification_time)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_assert (modification_time != NULL);
 
 	if (file == NULL)
@@ -460,24 +479,26 @@ _gtk_source_file_get_modification_time (GtkSourceFile *file,
 
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE (file), FALSE);
 
-	if (file->priv->modification_time_set)
+	if (priv->modification_time_set)
 	{
-		*modification_time = file->priv->modification_time;
+		*modification_time = priv->modification_time;
 	}
 
-	return file->priv->modification_time_set;
+	return priv->modification_time_set;
 }
 
 void
 _gtk_source_file_set_modification_time (GtkSourceFile *file,
-					GTimeVal       modification_time)
+					gint64         modification_time)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	if (file != NULL)
 	{
 		g_return_if_fail (GTK_SOURCE_IS_FILE (file));
 
-		file->priv->modification_time = modification_time;
-		file->priv->modification_time_set = TRUE;
+		priv->modification_time = modification_time;
+		priv->modification_time_set = TRUE;
 	}
 }
 
@@ -494,14 +515,16 @@ _gtk_source_file_set_modification_time (GtkSourceFile *file,
 gboolean
 gtk_source_file_is_local (GtkSourceFile *file)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE (file), FALSE);
 
-	if (file->priv->location == NULL)
+	if (priv->location == NULL)
 	{
 		return FALSE;
 	}
 
-	return g_file_has_uri_scheme (file->priv->location, "file");
+	return g_file_has_uri_scheme (priv->location, "file");
 }
 
 /**
@@ -523,14 +546,15 @@ gtk_source_file_is_local (GtkSourceFile *file)
 void
 gtk_source_file_check_file_on_disk (GtkSourceFile *file)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
 	GFileInfo *info;
 
-	if (file->priv->location == NULL)
+	if (priv->location == NULL)
 	{
 		return;
 	}
 
-	info = g_file_query_info (file->priv->location,
+	info = g_file_query_info (priv->location,
 				  G_FILE_ATTRIBUTE_TIME_MODIFIED ","
 				  G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
 				  G_FILE_QUERY_INFO_NONE,
@@ -539,25 +563,28 @@ gtk_source_file_check_file_on_disk (GtkSourceFile *file)
 
 	if (info == NULL)
 	{
-		file->priv->deleted = TRUE;
+		priv->deleted = TRUE;
 		return;
 	}
 
 	if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED) &&
-	    file->priv->modification_time_set)
+	    priv->modification_time_set)
 	{
-		GTimeVal timeval;
+		GDateTime *dt;
+		gint64 mtime;
 
-		g_file_info_get_modification_time (info, &timeval);
+		dt = g_file_info_get_modification_date_time (info);
+		mtime = g_date_time_to_unix (dt);
 
 		/* Note that the modification time can even go backwards if the
 		 * user is copying over an old file.
 		 */
-		if (timeval.tv_sec != file->priv->modification_time.tv_sec ||
-		    timeval.tv_usec != file->priv->modification_time.tv_usec)
+		if (mtime != priv->modification_time)
 		{
-			file->priv->externally_modified = TRUE;
+			priv->externally_modified = TRUE;
 		}
+
+		g_date_time_unref (dt);
 	}
 
 	if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
@@ -574,11 +601,13 @@ gtk_source_file_check_file_on_disk (GtkSourceFile *file)
 
 void
 _gtk_source_file_set_externally_modified (GtkSourceFile *file,
-					  gboolean       externally_modified)
+                                          gboolean       externally_modified)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_if_fail (GTK_SOURCE_IS_FILE (file));
 
-	file->priv->externally_modified = externally_modified != FALSE;
+	priv->externally_modified = externally_modified != FALSE;
 }
 
 /**
@@ -597,18 +626,22 @@ _gtk_source_file_set_externally_modified (GtkSourceFile *file,
 gboolean
 gtk_source_file_is_externally_modified (GtkSourceFile *file)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE (file), FALSE);
 
-	return file->priv->externally_modified;
+	return priv->externally_modified;
 }
 
 void
 _gtk_source_file_set_deleted (GtkSourceFile *file,
-			      gboolean       deleted)
+                              gboolean       deleted)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_if_fail (GTK_SOURCE_IS_FILE (file));
 
-	file->priv->deleted = deleted != FALSE;
+	priv->deleted = deleted != FALSE;
 }
 
 /**
@@ -627,23 +660,27 @@ _gtk_source_file_set_deleted (GtkSourceFile *file,
 gboolean
 gtk_source_file_is_deleted (GtkSourceFile *file)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE (file), FALSE);
 
-	return file->priv->deleted;
+	return priv->deleted;
 }
 
 void
 _gtk_source_file_set_readonly (GtkSourceFile *file,
-			       gboolean       readonly)
+                               gboolean       readonly)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_if_fail (GTK_SOURCE_IS_FILE (file));
 
 	readonly = readonly != FALSE;
 
-	if (file->priv->readonly != readonly)
+	if (priv->readonly != readonly)
 	{
-		file->priv->readonly = readonly;
-		g_object_notify (G_OBJECT (file), "read-only");
+		priv->readonly = readonly;
+		g_object_notify_by_pspec (G_OBJECT (file), properties[PROP_READ_ONLY]);
 	}
 }
 
@@ -663,7 +700,9 @@ _gtk_source_file_set_readonly (GtkSourceFile *file,
 gboolean
 gtk_source_file_is_readonly (GtkSourceFile *file)
 {
+	GtkSourceFilePrivate *priv = gtk_source_file_get_instance_private (file);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE (file), FALSE);
 
-	return file->priv->readonly;
+	return priv->readonly;
 }
